#include "framebuffer.h"
#include "font_alotware.h"
#include "serial.h"
#include "xtoa.h"

static unsigned long fb_address;
static unsigned long fb_pitch;
static unsigned short fb_width, fb_height;
static unsigned char fb_bpp, fb_bytespp;
static unsigned char fb_rpos, fb_bpos, fb_gpos;

void initfb(
	unsigned long addr,
	unsigned short w,
	unsigned short h,
	unsigned char bpp,
	unsigned long pitch,
	unsigned char rpos,
	unsigned char gpos,
	unsigned char bpos
){
	fb_address = addr;
	fb_pitch = pitch;
	fb_width = w;
	fb_height = h;
	fb_bpp = bpp;
	fb_bytespp = bpp/8;
	fb_rpos = rpos;
	fb_gpos = gpos;
	fb_bpos = bpos;
}

void putpixel(
	unsigned short x,
	unsigned short y,
	unsigned char r,
	unsigned char g,
	unsigned char b
){
	if (x > fb_width || y > fb_height)
		return;
	unsigned long dest_addr = fb_address + y*fb_pitch + x*fb_bytespp;
	if (fb_bpp == 32) {
		*((unsigned long *)(dest_addr)) = r<<fb_rpos | g<<fb_gpos | b<<fb_bpos;
	} else {
		*((unsigned char *)(dest_addr + fb_rpos/8)) = r;
		*((unsigned char *)(dest_addr + fb_gpos/8)) = g;
		*((unsigned char *)(dest_addr + fb_bpos/8)) = b;
	}
}

void putc(
	const char *character,
	unsigned short x,
	unsigned short y,
	unsigned char r,
	unsigned char g,
	unsigned char b
){
	// Limit to ASCII printable
	if (*character < 0x20 || *character > 0x7e) {
		serial_write_string("\nputc ASCII out of range: ");
		serial_write_string(uitoa(*character, 16));
		return;
	}

	unsigned char *rowdata = font[*character];
	unsigned char charx, chary, iy, ix;
	unsigned char sizex=1;
	unsigned char sizey=1;
	for (chary=0;chary<16;chary++) {
		for (charx=0;charx<8;charx++) {
			//unsigned char pix = rowdata[chary] & 1 << charx;
			unsigned char pix = rowdata[chary] & 0b10000000 >> charx;
			if (pix) {
				for (iy=0;iy<sizey;iy++) {
					for (ix=0;ix<sizex;ix++) {
						putpixel(x+charx*sizex+ix, y+chary*sizey+iy, r, g, b);
					}
				}
			}
		}
	}
}

void puts(
	const char *text,
	unsigned short x,
	unsigned short y,
	unsigned char r,
	unsigned char g,
	unsigned char b
){
	unsigned char i = 0;
	while (text[i]) {
		putc(text+i, x+i*8, y, r, g, b);
		i++;
	}
}
